@@ -1,3 +1,6 @@
+import asyncio
+
+from asgiref.sync import sync_to_async
 from django.db import models as django_models
 
 
@@ -10,25 +13,42 @@ def default_cache_func(
     redis_dicts = redis_root.get(redis_model, return_dict=True)
     # cache_to_django(
     #     django_model,
-    #     redis_dicts
+    #     redis_dicts,
+    #     redis_root,
     # )
     django_to_cache(
         django_model,
         redis_model,
         redis_root,
-        get_or_create_redis_model_from_django_model
+        get_or_create_redis_model_from_django_model,
     )
 
 
-def cache_to_django(django_model, redis_dicts):
-    for redis_instance_id, redis_instance_data in redis_dicts.items():
-        django_instance = update_or_create_django_instance_from_redis_instance(
-            redis_instance_data,
-            django_model,
-        )
+def cache_to_django(django_model, redis_dicts, redis_root):
+    async def run_cache_to_django(django_model, redis_dicts, redis_root):
+        async_chunk_size = redis_root.async_db_requests_limit
+        async_tasks = []
+        completed = 0
+        to_complete = len(redis_dicts.keys())
+
+        for redis_instance_id, redis_instance_data in redis_dicts.items():
+            async_tasks.append(
+                update_or_create_django_instance_from_redis_instance(
+                    redis_instance_data,
+                    django_model,
+                )
+            )
+            completed += 1
+            if len(async_tasks) == async_chunk_size or completed == to_complete:
+                await asyncio.gather(*async_tasks)
+                async_tasks = []
+                print(f'Recorded {completed} {django_model.__name__} instances from cache to django')
+
+    loop = asyncio.get_event_loop()
+    loop.run_until_complete(run_cache_to_django(django_model, redis_dicts, redis_root))
 
 
-def update_or_create_django_instance_from_redis_instance(
+async def update_or_create_django_instance_from_redis_instance(
         redis_instance,
         django_model,
 ):
@@ -62,7 +82,6 @@ def update_or_create_django_instance_from_redis_instance(
             django_id,
             django_many_to_many_params
         )
-        print(f'Added {django_instance.__class__.__name__}:{django_id} to django from cache')
     return django_instance
 
 
@@ -116,37 +135,63 @@ def django_to_cache(
         redis_root,
         get_or_create_redis_model_from_django_model
 ):
-    for django_instance in django_model.objects.all():
-        redis_instance = update_or_create_redis_instance_from_django_instance(
-            django_instance,
+    async def run_django_to_cache(
+            django_model,
             redis_model,
             redis_root,
-            get_or_create_redis_model_from_django_model
+            get_or_create_redis_model_from_django_model,
+    ):
+        django_instances = await sync_to_async(django_model.objects.all)()
+        async_chunk_size = redis_root.async_db_requests_limit
+        async_tasks = []
+        completed = 0
+        to_complete = len(django_instances)
+
+        for django_instance in django_instances:
+            async_tasks.append(
+                update_or_create_redis_instance_from_django_instance(
+                    django_instance,
+                    redis_model,
+                    redis_root,
+                    get_or_create_redis_model_from_django_model
+                )
+            )
+            completed += 1
+            if len(async_tasks) == async_chunk_size or completed == to_complete:
+                await asyncio.gather(*async_tasks)
+                async_tasks = []
+                print(f'Recorded {completed} {django_model.__name__} instances from django to cache')
+
+    loop = asyncio.get_event_loop()
+    loop.run_until_complete(
+        run_django_to_cache(
+            django_model,
+            redis_model,
+            redis_root,
+            get_or_create_redis_model_from_django_model,
         )
+    )
 
 
-def update_or_create_redis_instance_from_django_instance(
+async def update_or_create_redis_instance_from_django_instance(
         django_instance,
         redis_model,
         redis_root,
         get_or_create_redis_model_from_django_model
 ):
     redis_instance = redis_root.get(redis_model, django_id=django_instance.id)
-    print(django_instance)
     redis_dict = django_instance_to_redis_params(
         django_instance,
         redis_root,
         get_or_create_redis_model_from_django_model
     )
 
-
     if not redis_instance:
         redis_model(
             redis_root=redis_root,
             django_id=django_instance.id,
             **redis_dict
         ).save()
-        print(f'Added {django_instance.__class__.__name__}:{django_instance.id} to cache from django')
     else:
         redis_instance = redis_instance[0]
         redis_root.update(redis_model, redis_instance,
@@ -164,7 +209,6 @@ def django_instance_field_to_redis_value(
         redis_root,
         get_or_create_redis_model_from_django_model
 ):
-
     django_field_value = getattr(django_instance, django_field.name)
     redis_value = django_field_value
     if django_field.__class__ == django_models.ForeignKey:
@@ -222,7 +266,8 @@ def get_or_create_redis_instance_from_django_instance(
     redis_model = get_or_create_redis_model_from_django_model(django_instance_model, redis_root)
     redis_instance = redis_root.get(redis_model, django_id=django_instance.id)
     if not redis_instance:
-        redis_dict = django_instance_to_redis_params(django_instance, redis_root, get_or_create_redis_model_from_django_model)
+        redis_dict = django_instance_to_redis_params(django_instance, redis_root,
+                                                     get_or_create_redis_model_from_django_model)
         redis_model(
             redis_root=redis_root,
             django_id=django_instance.id,
