@@ -392,9 +392,10 @@ class RedisRoot:
             self,
             connection_pool=None,
             prefix='redis_test',
+            async_db_requests_limit=50,
             ignore_deserialization_errors=True,
             save_consistency=False,
-            economy=False
+            economy=False,
     ):
         self.registered_models = []
         self.registered_django_models = {}
@@ -404,6 +405,7 @@ class RedisRoot:
             print(f'Prefix {prefix} is type of {type(prefix)}, allowed only str, using default prefix "redis_test"')
             self.prefix = 'redis_test'
         self.connection_pool = self._get_connection_pool(connection_pool)
+        self.async_db_requests_limit = async_db_requests_limit
         self.ignore_deserialization_errors = ignore_deserialization_errors
         self.save_consistency = save_consistency
         self.economy = economy
